/* $LAAS: viamlib.h 2008/07/01 15:32:17 mallet $
 *
 * Copyright (c) 2007-2008 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *                                      Anthony Mallet on Mon Jan 22 2007
 */
#ifndef H_VIAMLIB
#define H_VIAMLIB

#include <sys/time.h>
#include <stdint.h>

#include "viamtypes.h"


/* --- Library generic routines -------------------------------------- */

struct _IplImage; /**< forward decl to avoid including cxtypes here */
typedef struct viam_handle_t *	viam_handle_t;

viam_handle_t		viam_init(void);
void			viam_release(viam_handle_t vh);
viam_errorcode_t	viam_errno(viam_handle_t vh);


/* --- Bank and camera configuration --------------------------------- */

typedef struct viam_bank_t *	viam_bank_t;
typedef struct viam_camera_t *	viam_camera_t;
typedef struct viam_image_t *	viam_image_t;

viam_bank_t	viam_bank_create(viam_handle_t vh, const char *name);
void		viam_bank_delete(viam_handle_t vh, viam_bank_t bank);
int		viam_bank_cameraadd(viam_handle_t vh, viam_bank_t bank,
			const viam_camera_t camera, const char *name);
const char *	viam_bank_name(viam_bank_t bank);
int		viam_bank_nimages(viam_bank_t bank);
int		viam_bank_ncameras(viam_bank_t bank);
viam_image_t	viam_bank_image(viam_handle_t vh, viam_bank_t bank, int i);
viam_camera_t	viam_bank_camera(viam_handle_t vh, viam_bank_t bank, int i);
void *		viam_bank_usrdata(viam_bank_t bank);
void		viam_bank_setusrdata(viam_bank_t bank, void *usrdata);

viam_camera_t	viam_camera_create(viam_handle_t vh, const char *uid,
			const char *name);
void		viam_camera_delete(viam_handle_t vh, viam_camera_t camera);
const char *	viam_camera_name(viam_camera_t cam);
const char *	viam_camera_uid(viam_camera_t c);
const viam_hwmode_t *
		viam_camera_hwmode(viam_handle_t vh, viam_camera_t camera);
int		viam_camera_sethwmode(viam_handle_t vh, viam_camera_t camera,
			const viam_hwmode_t *hwmode);
void *		viam_camera_usrdata(viam_camera_t camera);
void		viam_camera_setusrdata(viam_camera_t camera, void *usrdata);

const char *	viam_image_name(viam_image_t image);
viam_camera_t	viam_image_camera(const viam_image_t image);
int		viam_image_geometry(viam_handle_t vh, const viam_image_t image,
			viam_geometry_t *geom);

viam_bank_t	viam_bank_getbyname(const viam_handle_t vh, const char *name);
viam_camera_t	viam_camera_getbyuid(const viam_handle_t vh, const char *uid);
viam_camera_t	viam_camera_getbyname(const viam_handle_t vh, const char *name);
viam_image_t	viam_image_getbyname(const viam_handle_t vh, const char *name);


/* --- Hardware control ---------------------------------------------- */

int		viam_hardware_list(viam_handle_t vh);

int		viam_hardware_attach(viam_handle_t vh);
void		viam_hardware_detach(viam_handle_t vh);

int		viam_bank_configure(viam_handle_t vh, viam_bank_t bank);
int		viam_bank_unconfigure(viam_handle_t vh, viam_bank_t bank);
int		viam_bank_isconfigured(viam_bank_t bank);

int		viam_queryhwmodes(viam_handle_t vh, viam_camera_t camera,
			viam_hwmode_t *supported, size_t n);
int		viam_hwmode_size(viam_handle_t vh, const viam_hwmode_t *hwmode,
			int *width, int *height, int *depth, int *channels);

int		viam_datatransmit(viam_handle_t vh, viam_bank_t bank,
			viam_onoff_t on);
int		viam_oneshot(viam_handle_t vh, viam_bank_t bank,
			struct _IplImage *image[], struct timeval *timestamps[],
			int nimages);


/* --- Image filters ------------------------------------------------- */

typedef struct viam_filter_t *		viam_filter_t;

int		viam_queryfeature(viam_handle_t vh, viam_camera_t camera,
			viam_filtertype_t filter, unsigned int *methods,
			unsigned int *automodes);

viam_filter_t	viam_filter_format_create(viam_handle_t vh, const char *name,
			viam_filtertype_t type, viam_filterformat_t format);
int		viam_filter_format_update(viam_handle_t vh, viam_filter_t filter,
			viam_filtermethod_t method, viam_filterautomode_t aumode,
			viam_filterformat_t format);
viam_filter_t	viam_filter_geometry_create(viam_handle_t vh,
			const char *name, viam_filtertype_t type, double left,
			double top, double right, double bottom, double towidth,
			double toheight);
int		viam_filter_geometry_update(viam_handle_t vh,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode, double left, double top,
			double right, double bottom, double towidth,
			double toheight);
viam_filter_t	viam_filter_luminance_create(viam_handle_t vh,
			const char *name, viam_filtertype_t type,
			double parameter);
int		viam_filter_luminance_update(viam_handle_t vh,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode, double parameter);
viam_filter_t	viam_filter_color_create(viam_handle_t vh, const char *name,
			viam_filtertype_t type, double parameter);
int		viam_filter_color_update(viam_handle_t vh,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode, double parameter);
viam_filter_t	viam_filter_improc_create(viam_handle_t vh, const char *name,
			viam_filtertype_t type, double parameter);
int		viam_filter_improc_update(viam_handle_t vh,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode, double parameter);

int		viam_filter_push(viam_handle_t vh, viam_image_t camera,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t automode);

viam_filter_t	viam_filter_getbyname(viam_handle_t vh, const char *name);

const char *		viam_filter_name(const viam_filter_t filter);
viam_filtertype_t	viam_filter_type(const viam_filter_t filter);
viam_filtermethod_t	viam_filter_method(const viam_filter_t filter);
viam_filterautomode_t	viam_filter_autoarg(const viam_filter_t filter);


/* --- Camera calibration -------------------------------------------- */

#ifdef VIAM_OPENCV_SUPPORT
int		viam_calibration_start(viam_handle_t vh, viam_bank_t bank,
			viam_caltype_t type, int rows, int cols, double size);
int		viam_calibration_addimage(viam_handle_t vh, viam_bank_t bank,
			struct _IplImage *image[], int nimages, double distth,
			viam_onoff_t feedback, int *result);
int		viam_calibration_estimate(viam_handle_t vh, viam_bank_t bank,
			double *result);
int		viam_calibration_finish(viam_handle_t vh, viam_bank_t bank);
#endif /* VIAM_OPENCV_SUPPORT */

int		viam_image_calibration(viam_handle_t vh, viam_image_t image,
			viam_cameracalibration_t *cal, const viam_geometry_t *geom);
int		viam_bank_calibration(viam_handle_t vh, viam_bank_t bank,
			viam_bankcalibration_t *cal);

int		viam_calibration_save(viam_handle_t vh, viam_bank_t bank,
			const char *file);
int		viam_calibration_load(viam_handle_t vh, viam_bank_t bank,
			const char *file);


/* --- Image structure ----------------------------------------------- */

#ifndef VIAM_OPENCV_SUPPORT

/* Provide our simple replacement for the missing opencv structures when
 * configured without opencv support. Note that the IplImage provided
 * here is not compatible with opencv's one. */

typedef struct _IplImage {
  int  nChannels;		/**< 1 or 3 channels */
  int  depth;			/**< pixel depth in bits */
  int  width;			/**< image width in pixels */
  int  height;			/**< image height in pixels */
  int  imageSize;		/**< image data size in bytes */
  unsigned char *imageData;	/**< pointer to aligned image data */
  int  widthStep;		/**< size of aligned image row in bytes */
} IplImage;

typedef struct CvSize {
  int width;
  int height;
} CvSize;

#define IPL_DEPTH_8U	8
#define IPL_DEPTH_16U	16

#define IPL_ORIGIN_TL	0

#define CV_AUTOSTEP	0x7fffffff

#define CV_LOAD_IMAGE_UNCHANGED  -1

CvSize		cvSize(int width, int height);
IplImage *	cvCreateImage(CvSize size, int depth, int channels);
IplImage *	cvCreateImageHeader(CvSize size, int depth, int channels);
IplImage *	cvInitImageHeader(IplImage *image, CvSize size, int depth,
			int channels, int origin, int align);
void		cvSetData(struct _IplImage *image, void *data, int step);
int		cvSaveImage(const char *filename, const struct _IplImage *image);
IplImage *	cvLoadImage(const char* filename, int iscolor);
void		cvReleaseImage(IplImage **image);

#endif /* !VIAM_OPENCV_SUPPORT */

#endif /* H_VIAMLIB */
