/* $LAAS: viam_private.h 2008/07/01 15:57:23 mallet $
 *
 * Copyright (c) 2007-2008 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *                                      Anthony Mallet on Tue Jan 23 2007
 */
#ifndef H_VIAM_PRIVATE
#define H_VIAM_PRIVATE

#include "viamlib.h"

/* --- Forward decls ------------------------------------------------- */

typedef struct viam_hwdev_t viam_hwdev_t;

typedef struct viam_bankcalib_t *viam_bankcalib_t;
typedef struct viam_cameracalib_t *viam_cameracalib_t;


/* --- Library generic routines -------------------------------------- */

int		viam_seterrno(viam_handle_t vh, int eno);

viam_camera_t	viam_config_cameras(viam_handle_t vh);
void		viam_config_setcameras(viam_handle_t vh, viam_camera_t camera);

viam_bank_t	viam_config_banks(viam_handle_t vh);
void		viam_config_setbanks(viam_handle_t vh, viam_bank_t bank);

viam_image_t	viam_config_images(viam_handle_t vh);
void		viam_config_setimages(viam_handle_t vh, viam_image_t image);

viam_filter_t	viam_config_filters(viam_handle_t vh);
void		viam_config_setfilters(viam_handle_t vh, viam_filter_t filter);


/* --- Bank and camera configuration --------------------------------- */

viam_bank_t		viam_bank_next(viam_bank_t camera);
viam_bankcalib_t	viam_bank_caldata(viam_bank_t bank);
void			viam_bank_setcaldata(viam_bank_t bank,
				viam_bankcalib_t c);
void			viam_bank_setconfigured(viam_bank_t bank,
				int configured);
void *			viam_bank_hwdata(viam_bank_t bank);
void			viam_bank_sethwdata(viam_bank_t bank, void *hwdata);

viam_camera_t		viam_camera_next(viam_camera_t camera);
viam_hwdev_t *		viam_camera_hwdev(viam_camera_t camera);
void			viam_camera_sethwdev(viam_camera_t camera,
				viam_hwdev_t *hwdev);
int			viam_camera_setup(viam_camera_t camera);
void			viam_camera_setsetup(viam_camera_t camera,
				int setup);
struct _IplImage *	viam_camera_raw(viam_camera_t camera);
void *			viam_camera_hwdata(viam_camera_t camera);
void			viam_camera_sethwdata(viam_camera_t camera,
				void *hwdata);

viam_image_t		viam_image_create(viam_handle_t vh, const char *name,
				viam_camera_t camera);
viam_image_t		viam_image_next(viam_image_t image);
viam_filter_t		viam_image_flist(const viam_image_t image);
void			viam_image_setflist(viam_image_t image,
				viam_filter_t flist);
viam_cameracalib_t	viam_image_caldata(const viam_image_t image);
void			viam_image_setcaldata(viam_image_t image,
				viam_cameracalib_t c);

void *			viam_swap(void);
void			viam_setswap(size_t size);


/* --- Image filters ------------------------------------------------- */

typedef union viam_filterargs_t {
  struct /* anonymous */ {
    double left, top, right, bottom;
    double towidth, toheight;
  };					/**< crop, scale and the like */
  struct {
    double blue, red;			/**< white balance */
  };
  double value;				/**< single valued filters */
  viam_filterformat_t format;		/**< for format filters */
} viam_filterargs_t;

int	viam_filter_inithw(viam_handle_t vh, viam_image_t image);
int	viam_filter_image(viam_handle_t vh, viam_image_t camera,
		struct _IplImage *final);
int	viam_filter_configure(viam_handle_t vh, viam_image_t camera,
		viam_filter_t f);
int	viam_filter_geo(viam_handle_t vh, viam_filter_t f,
		viam_geometry_t *geom);
int	viam_filter_queryswfeature(viam_handle_t vh, viam_filtertype_t filter,
		unsigned int *methods, unsigned int *aumodes);

int	viam_format_geosetup(viam_handle_t vh, viam_image_t image,
		viam_filterargs_t *args, viam_geometry_t *geom, void **data);
int	viam_cvtformat(viam_handle_t vh, viam_image_t image,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_t *args, void *data);

int	viam_geo_subsample(viam_handle_t vh, viam_image_t camera,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_t *args, void *data);
int	viam_geo_geosetup(viam_handle_t vh, viam_image_t camera,
		viam_filterargs_t *args, viam_geometry_t *geom, void **data);
#ifdef VIAM_OPENCV_SUPPORT
int	viam_geo_remap(viam_handle_t vh, viam_image_t camera,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_t *args, void *data);
int	viam_geo_distortsetup(viam_handle_t vh, viam_image_t camera,
		viam_filterargs_t *args, viam_geometry_t *geom, void **data);
int	viam_geo_rectifysetup(viam_handle_t vh, viam_image_t camera,
		viam_filterargs_t *args, viam_geometry_t *geom, void **data);
#endif /* VIAM_OPENCV_SUPPORT */

int	viam_luminance_geosetup(viam_handle_t vh, viam_image_t image,
		viam_filterargs_t *args, viam_geometry_t *geom, void **data);

int	viam_color_geosetup(viam_handle_t vh, viam_image_t camera,
		viam_filterargs_t *args, viam_geometry_t *geom, void **data);
int	viam_awb(viam_handle_t vh, viam_image_t camera,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_t *args, void *data);
int	viam_awb_auto(viam_handle_t vh, viam_image_t camera,
		const struct _IplImage *source, viam_filterautomode_t *aumode,
		viam_filterargs_t *args, void *data);

#ifdef VIAM_OPENCV_SUPPORT
int	viam_improc_log(viam_handle_t vh, viam_image_t camera,
		const struct _IplImage *source, struct _IplImage *result,
		viam_filterargs_t *args, void *data);
#endif /* VIAM_OPENCV_SUPPORT */


/* --- Hardware bus access ------------------------------------------- */

/* Generic */
const char *	viam_hwdev_name(viam_hwdev_t *hwdev);
viam_hwdev_t *	viam_hardware_device(viam_handle_t vh, const char *id,
			char **uid);
int		viam_hardware_init(viam_handle_t vh);
int		viam_sethwfeaturemode(viam_handle_t vh, viam_camera_t camera,
			viam_filtertype_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode);
int		viam_sethwfeaturevalue(viam_handle_t vh, viam_camera_t camera,
			viam_filtertype_t filter,
			const viam_filterargs_t *args);

/* IEEE1394 bus */
#ifdef VIAM_DC1394_SUPPORT
int	viam1394_init(viam_handle_t vh);
int	viam1394_fini(viam_handle_t vh);

int	viam1394_hardware_list(viam_handle_t vh);

int	viam1394_hardware_attach(viam_handle_t vh, viam_camera_t camera);
void	viam1394_hardware_detach(viam_handle_t vh, viam_camera_t camera);

int	viam1394_camera_configure(viam_handle_t vh, viam_camera_t camera);
int	viam1394_camera_unconfigure(viam_handle_t vh, viam_camera_t camera);

int	viam1394_datatransmit(viam_handle_t vh, viam_camera_t camera,
		viam_onoff_t on);

int	viam1394_capture(viam_handle_t vh, viam_camera_t camera);
char *	viam1394_capturedata(viam_handle_t vh, viam_camera_t camera, size_t *bytes,
		size_t *rowstride);
struct timeval	viam1394_capturetimestamp(viam_handle_t vh,
			viam_camera_t camera);
int	viam1394_capturedone(viam_handle_t vh, viam_camera_t camera);

int	viam1394_queryhwmodes(viam_handle_t vh, viam_camera_t camera,
		viam_hwmode_t *supported, size_t n);
int	viam1394_queryhwfeature(viam_handle_t vh, viam_camera_t camera,
		viam_filtertype_t filter, unsigned int *methods, unsigned int *aumodes);
int	viam1394_sethwfeaturemode(viam_handle_t vh, viam_camera_t camera,
		viam_filtertype_t filter, viam_filtermethod_t method,
		viam_filterautomode_t aumode);
int	viam1394_sethwfeaturevalue(viam_handle_t vh, viam_camera_t camera,
		viam_filtertype_t filter, const viam_filterargs_t *args);
#endif /* VIAM_DC1394_SUPPORT */

/* acquisition from files */
int	viamfile_hardware_list(viam_handle_t vh);

int	viamfile_hardware_attach(viam_handle_t vh, viam_camera_t camera);
void	viamfile_hardware_detach(viam_handle_t vh, viam_camera_t camera);

int	viamfile_camera_configure(viam_handle_t vh, viam_camera_t camera);
int	viamfile_camera_unconfigure(viam_handle_t vh, viam_camera_t camera);

int	viamfile_datatransmit(viam_handle_t vh, viam_camera_t camera,
		viam_onoff_t on);

int	viamfile_capture(viam_handle_t vh, viam_camera_t camera);
char *	viamfile_capturedata(viam_handle_t vh, viam_camera_t camera, size_t *bytes,
		size_t *rowstride);
struct timeval	viamfile_capturetimestamp(viam_handle_t vh,
			viam_camera_t camera);
int	viamfile_capturedone(viam_handle_t vh, viam_camera_t camera);

int	viamfile_queryhwmodes(viam_handle_t vh, viam_camera_t camera,
		viam_hwmode_t *supported, size_t n);
int	viamfile_queryhwfeature(viam_handle_t vh, viam_camera_t camera,
		viam_filtertype_t filter, unsigned int *methods, unsigned int *aumodes);
int	viamfile_sethwfeaturemode(viam_handle_t vh, viam_camera_t camera,
		viam_filtertype_t filter, viam_filtermethod_t method,
		viam_filterautomode_t aumode);
int	viamfile_sethwfeaturevalue(viam_handle_t vh, viam_camera_t camera,
		viam_filtertype_t filter, const viam_filterargs_t *args);


/* --- Image calibration --------------------------------------------- */

void	viam_initrectifmap(const double a[9], const double distort[4],
		const double rectif[9], float *mapx, float *mapy,
		int width, int height);

#endif /* H_VIAM_PRIVATE */
